# -*- coding: utf-8 -*-
# xDevs.com Confidential
# 
# $Id: main.py | Rev 26  | 2018/10/10 06:35:25 tin_fpga $
# xDevs.com TEC Experiment setup
# 
# Copyright (c) 2012-2017, xDevs.com
# 
# Python 2.7 | RPI3 | Win32
# Project maintainers:
#  o Tsemenko Ilya  (@)
# 
# https://xdevs.com/guide/teckit
#
import os
import mmap
import sys
#import Gpib
import time
import ftplib
import numbers
import signal
#import k7168_client
import ConfigParser
import numpy as np
from x256 import x256

cfg = ConfigParser.ConfigParser()
cfg.read('teckit.conf')
cfg.sections()

#if cfg.get('teckit', 'if_debug', 1) == 'true':
#    from if_debug import *
#elif cfg.get('teckit', 'if_debug', 1) == 'false':
#    if cfg.get('teckit', 'interface', 1) == 'gpib':
#        from if_gpib import *
#    else:
#        print "No interface defined!"
#        quit()

print "\033[2J"

# Device Libs

import imp 
if (cfg.get('mode', 'no_thermal', 1) == "false"):
    k2510  = imp.load_source('k2510' , 'devices/k2510.py')    # Load support for K2510
dmm1   = imp.load_source('hp3458', 'devices/hp3458.py')   # Load support for HP3458A
dmm2   = imp.load_source('hp3458', 'devices/hp3458.py')   # Load support for HP3458A
dmm3   = imp.load_source('hp3458', 'devices/hp3458.py')   # Load support for HP3458A
#dmm4   = imp.load_source('hp3459', 'devices/hp3459.py')   # Load support for HP3458A
#dmm5   = imp.load_source('f8508a', 'devices/f8508a.py')   # Load support for HP3458A
#dmm2   = imp.load_source('hp3458', 'devices/hp3458.py')   # Load support for HP3458A
#dmm3   = imp.load_source('hp34401' , 'devices/hp34401.py')    # Load support for K2002
#dmm4   = imp.load_source('k2002' , 'devices/k2002.py')    # Load support for K2002
#em1 = imp.load_source('hp53131' , 'devices/hp53131a.py')    # Load support for K6517
#dmm5   = imp.load_source('r6581t' , 'devices/r6581t.py')    # Load support for R6581T
#ilx5910 = imp.load_source('ilx5910', 'devices/ilx.py')    # Load support for ILX 5910B
#dmm1   = imp.load_source('k2182' , 'devices/k2182.py')    # Load support for K2002
#dmm6   = imp.load_source('k2182' , 'devices/k2182.py')    # Load support for K2002
#dmm7   = imp.load_source('k182m' , 'devices/k182m.py')    # Load support for K182M
#mfc   = imp.load_source('f5700', 'devices/f5700a.py')    # Load support for F5700A

ext_temp     = 0.0
ext_rh       = 0.0
ext_pressure = 0.0

if cfg.get('teckit', 'env_sensor', 1) == 'bme280':
    from Adafruit_BME280 import *
    env_sensor = BME280(mode=BME280_OSAMPLE_8)
    def read_environment():
        global ext_temp, ext_rh, ext_pressure
        ext_temp     = env_sensor.read_temperature()
        ext_rh       = env_sensor.read_humidity()
        ext_pressure = env_sensor.read_pressure() / 100
else:
    def read_environment():
        global ext_temp, ext_rh, ext_pressure
        ext_temp     = 0
        ext_rh       = 0
        ext_pressure = 0    

root_dir = ''
fileName4 = root_dir + cfg.get('teckit', 'data_file', 1)

# Experiment settings
'''
               Peak_temp
               ______
              /      \
             | Step   \
            /          \
   Slope_pos            \ Slope_neg
          /              \
time_start Speed_pos / neg\  time_end
  ------/                  \--------- TEC temp
 Sv_start                    Sv_end
   2h     9h     2h    9h       2h
'''

sv_start            = float(cfg.get('testset', 'sv_start', 1)) # Chamber start temperature
sv_end              = float(cfg.get('testset', 'sv_end', 1))  # Chamber end temperature
peak_temp           = float(cfg.get('testset', 'peak_temp', 1))  # Top soak temperature
tps                 = int(cfg.get('testset', 'tps', 1))      # Sample Ratio for speed. 1 second = 1 index.
second_span         = int(cfg.get('testset', 'second_span', 1))
slope_pos           = float(cfg.get('testset', 'slope_pos', 1)) * second_span  # Steps for the positive slope
slope_neg           = float(cfg.get('testset', 'slope_neg', 1)) * second_span # Symmetrical negative slope
time_start          = float(cfg.get('testset', 'time_start', 1)) * second_span  # Initial hold temperature time, before positive slope starts
time_hold           = float(cfg.get('testset', 'time_hold', 1)) * second_span  # Hold temperature time once reached peak_temp
time_end            = float(cfg.get('testset', 'time_end', 1)) * second_span  # Hold temperature once rampdown finished

reference           = float(cfg.get('dut', 'reference', 1))  # Reference value
reference2          = float(cfg.get('dut2', 'reference', 1))  # Reference value
reference3          = float(cfg.get('dut3', 'reference', 1))  # Reference value
reference4          = float(cfg.get('dut4', 'reference', 1))  # Reference value
reference5          = float(cfg.get('dut5', 'reference', 1))  # Reference value
reference6          = float(cfg.get('dut6', 'reference', 1))  # Reference value
ppm_delta           = 0.0
ppm_delta2           = 0.0

pid_kp              = float(cfg.get('pid', 'kp', 1))  # P
pid_ki              = float(cfg.get('pid', 'ki', 1))  # I
pid_kd              = float(cfg.get('pid', 'kd', 1))  # D

step                = (peak_temp - sv_start) / slope_pos # Resolution for TEC temperature setting, in C
total_time          = time_start + time_hold + time_end + slope_pos + slope_neg # Test time, in seconds
elapsed_time        = 0
remaining_time      = 0
idx                 = 0       # current index
sv_temp             = sv_start
pv_temp             = 0.0     # current process value
tec_curr            = 0.0     # TEC Current

meas_val	    = 0.0     # DMM1 reading
meas_val2	    = 0.0     # DMM1 reading
meas_val3	    = 0.0     # DMM1 reading
meas_val4	    = 0.0     # DMM1 reading
meas_val5a	    = 0.0     # DMM1 reading
meas_val5b	    = 0.0     # DMM1 reading
meas_val6	    = 0.0     # DMM1 reading
meas_pcb_temp       = 0.0     # DMM2 reading

#scan = k7168_client.THP_socket('192.168.1.114',10001) 

w, h = 8, 1000;
ch_data = [[0 for x in range(w)] for y in range(h)] 
ew, eh, ech = 3, 1000, 8;
env_data = [[[0 for x in range(ew)] for y in range(eh)] for z in range(ech)] 
dc, dh = 8, 1000;
tsp_data = [[0 for x in range(dc)] for y in range(dh)] 

val2 = 0.0
idx = 0
tread = 20
temp2 = 0
nohm = 0
rtd_temp = 0.0
prev_t = 0
temp = 18
res_rtd = 0.3
exttemp = 25.0
rh = 50.0
pascals = 100000.00
hectopascals = 1000.00
tset = 18.0

# Check if file exists, if not create it and add header
def create_local_file(fileName):
    if (os.path.isfile(fileName) == False):
        with open(fileName, 'a') as o:
            o.write("date;hp3458a;hp3458b;k6;k4;meas5;meas6;k2001;temp1;temp2;amb_temp;amb_rh;amb_pressure;box_temp;nvm_temp;\r\n")
            print ("\033[2;40H-i- DataFile %s does not exist\r\n" % fileName) 
    else: 
        print ("\033[2;40H-i- Datafile %s exists\r\n" % fileName)

# Create file for the experiment and write header if needed
create_local_file(fileName4)

#time.sleep(100)
enable_start = 0
enable_hold = 0
reverse_dir = 0

print "\033[1;1H╒\033[1;100H╕\033[42;1H╘\033[42;100H╛"
for gi in range (2,100):
    print ("\033[1;%dH═" % (gi))
    print ("\033[3;%dH─" % (gi) )
    print ("\033[8;%dH═" % gi)
    print ("\033[40;%dH═" % gi)
    print ("\033[42;%dH═" % gi)
for gy in range (2,42):
    print ("\033[%d;1H│\033[%d;100H│" % (gy, gy))
print "\033[40;1H╞\033[8;1H╞\033[3;1H├\033[3;100H┤\033[8;100H╡\033[40;100H╡"
print "\033[41;66H│"

print ("\033[42;100H\r\n")
ix = x256.from_rgb(160, 130, 10)
print "\033[2;3H\x1b[38;5;" + str(ix) + "m xDevs.com TEC Experiment kit \033[0;49m"

tec_status = ["Hold start","Ramp up   ","Hold peak ","Ramp down ","Hold end  ","\033[41;1m !ALERT! ","\033[42;1m Job done"]
dmm_status = ["Configure ","Sample ","ACAL DCV ","ACAL ALL ","TEMP?   ","\033[41;1m !ERROR! ","\033[42;1m Job done"]
dmm_mode = ["DCV", "OHM", "OHMF", "DCI", "ACV", "ACI"]
dmm_terminal = ["\033[0;44m  FRONT  ", "\033[0;45m  REAR   "]

#print "\033[6;5H \033[0;31mGPIB[\033[1m%2d\033[0;31m] : Keithley 2002\033[0;39m" % 4
#print "\033[7;5H \033[0;33mGPIB[\033[1m%2d\033[0;33m] : Fluke 5700A  \033[0;39m" % 1

#tec = k2510.tec_meter(5,0,"TEC_SMU")
print "\033[9;72H \033[0;32mSet Temp     : %2.3f %cC\033[0;39m" % (25.0, u"\u00b0")
print "\033[10;72H \033[1;32mProcess Temp : %2.3f %cC\033[0;39m" % (25.0, u"\u00b0")
print "\033[11;72H \033[1;35mTEC Current  : %5.4f  A\033[0;39m" % (25.0)
print "\033[12;72H \033[1;33mStatus       : %s\033[0;39m" % (tec_status[5])
print "\033[13;72H \033[0;33mGain         : %9.4f \033[0;39m" % (pid_kp)
print "\033[14;72H \033[0;36mIntergal     : %9.4f \033[0;39m" % (pid_ki)
print "\033[15;72H \033[0;31mDerivative   : %9.4f \033[0;39m" % (pid_kd)

print "\033[9;40H \033[1;34mMeter mode   : %s \033[0;39m" % (dmm_mode[0])
print "\033[10;40H \033[1;38mMeasured val :%11.8g\033[0;39m" % (1000.04323)
print "\033[11;40H \033[0;32mOCOMP/DELAY  : %d, %d sec\033[0;39m" % (1, 0)
print "\033[12;40H \033[0;32mFixed range  : %s\033[0;39m" % (dmm_status[0])
print "\033[13;40H \033[0;36mNPLC         : %9.4f \033[0;39m" % (100)
print "\033[14;40H \033[0;37mTerminals    : %s \033[0;39m" % (dmm_terminal[0])
print "\033[15;40H \033[0;37mREL Value    :%11.8G \033[0;39m" % (1e-6)

print "\033[11;2H \033[1;32mMin temp     : %.3f %cC\033[0;39m" % (sv_start, u"\u00b0")
print "\033[12;2H \033[1;32mPeak temp    : %.3f %cC\033[0;39m" % (peak_temp, u"\u00b0")

print "\033[41;5H \033[0;35mREF :%11.9G \033[0;39m" % (reference)
print "\033[41;35H \033[1;35mDelta :%11.3f  ppm \033[0;39m" % (ppm_delta)

print "\033[42;0H"

if (cfg.get('mode', 'no_thermal', 1) == "false"):
    tecsmu = k2510.tec_meter(25,0,"2510")
    tecsmu.set_tmp("%5.3f" % sv_start)
    tecsmu.on_temp()
    tecsmu.set_gain(500.0)
    tecsmu.set_intg(5.00)
    tecsmu.set_derv(3.000)

#cntr = em1.cntr(3,0,"53131A")
dmm1 = dmm1.dmm_meter(2,0,"3458B")
dmm2 = dmm2.dmm_meter(3,0,"3458A")
dmm3 = dmm3.dmm_meter(10,0,"3458D")
#dmm4 = dmm4.dmm_meter(11,0,"3458C")
#dmm3 = dmm3.scpi_meter(8,0,"34401a")
#dmm4 = dmm4.scpi_meter(4,0,"2002-4")
#dmm5 = dmm5.flk_meter(5,0,"8508")
#dmm7 = dmm7.k182m_meter(18,0,"2182")

dmm1.set_dcv_range(1) #3458-7
dmm2.set_dcv_range(10) #3458-3
dmm3.set_ohm_range(10e3) #3458-3
#dmm4.set_dcv_range(10) # 3459b
#dmm2.set_dcv_range(10) # 3459b
#dmm3.set_dcv_range(10) # 3459b
#dmm4.set_dcv_range(1) # 3459b
#dmm3.set_dcv_range(10) # K2002-6
#dmm4.set_dcv_range(10) # K2002-4
#dmm5.set_dcv_range(10) # 8508
#dmm1.set_dci_range(1e-3) #2
#dmm1.inst.write("OCOMP 1")
#dmm3.set_ohm_range(200e3) #6
#dmm4.set_pt1000_rtd() #4
#dmm1.set_d_range(10) #4
#dmm2.set_dcv_range(10) #4

icnt = 0
time_ratio = 4.315             # Actual ratio to realworld processing speed (1 sample = 4.315 seconds)
dmm1_temp = dmm1.get_temp()
dmm2_temp = dmm2.get_temp()
dmm3_temp = dmm3.get_temp()
dmm4_temp = 24#dmm4.get_temp()
tread = 100

if (cfg.get('mode', 'run_acal', 1) == "true"):
#    dmm1.inst.write("ACAL ALL")
    #dmm2.inst.write("ACAL ALL")
#    dmm2.inst.write("ACAL ALL")
    time.sleep(860)

if (cfg.get('mode', 'run_acal_dcv', 1) == "true"):
    dmm1.inst.write("ACAL DCV")
    dmm2.inst.write("ACAL DCV")
#    dmm3.inst.write("ACAL DCV")
    time.sleep(150)

#dmm5 = dmm5.scpi_meter(10,0,"6581T")

sdev_arr = []
sdev_arr5 = []

while (idx <= (total_time * tps) ):
    global pv_temp  

    if cfg.get('teckit', 'env_sensor', 1) != 'none':
	ext_temp     = env_sensor.read_temperature()
	ext_rh       = env_sensor.read_humidity()
        ext_pressure = env_sensor.read_pressure() / 100
        print "\033[41;67H \033[1;31m%2.3f%cC  \033[1;32m%3.1f%%RH  \033[1;33m%4.1f hPa\033[0;39m" % (ext_temp, u"\u00b0", ext_rh, ext_pressure)
    else:
	ext_temp     = 24.0
	ext_rh       = 0
        ext_pressure = 0

    print "\033[9;2H \033[44;32m\033[1mSample       : %8d   \033[0;39m" % (idx)
    print "\033[10;2H \033[1;38mNext temp    : %.3f %cC\033[0;39m" % (sv_temp, u"\u00b0")
    
    rm, rs = divmod((total_time - idx) * time_ratio, 60)
    rh, rm = divmod(rm, 60)
    print "\033[13;2H \033[1;35mRemaining    : %2dh %02dm %02ds \033[0;39m" % (rh, rm, rs)
    em, es = divmod((idx) * time_ratio, 60)
    eh, em = divmod(em, 60)
    print "\033[14;2H \033[1;35mElapsed time : %2dh %02dm %02ds \033[0;39m" % (eh, em, es)
    print "\033[15;2H \033[0;37mProgress     :    %3.2f%% \033[0;39m" % ((float(idx) / float(total_time) ) * 100)
    
    if (icnt >= 22):
        icnt = 0
    print ("\033[%d;3H [%6d] : S%5.3f P%5.3f I%5.3f \033[1;34m A%12.9e \033[0;32mB%12.9e \033[0;33mC%12.9e C\033[0;39m" % (icnt+18, idx / tps, sv_temp, pv_temp, tec_curr, meas_val, meas_val2, meas_val3) ) 

    # Sample and mode sequencer
    if (idx <= time_start):
        # Holding for start
        sv_temp = sv_start
        print "\033[12;88H \033[1;33m%s\033[0;39m" % (tec_status[0])

    elif ((idx >= time_start) and (idx < (time_start + slope_pos) ) ):
        # Start positive slope ramp
        sv_temp = sv_start + (float((idx - time_start) * step)) 
        print "\033[12;88H \033[1;33m%s\033[0;39m" % (tec_status[1])

    elif ((idx >= (time_start + slope_pos)) and (idx < (time_start + slope_pos + time_hold))):
        # Start peak hold temp
        sv_temp = peak_temp
        print "\033[12;88H \033[1;33m%s\033[0;39m" % (tec_status[2])
    elif ((idx >= (time_start + slope_pos + time_hold)) and (idx < (time_start + slope_pos + time_hold + slope_neg))):
        # Ramp down
        sv_temp = peak_temp - (float(((idx) - (slope_neg + time_hold + time_start))) * step)
        pv_temp = idx
        print "\033[12;88H \033[1;33m%s\033[0;39m" % (tec_status[3])
    elif ((idx >= (time_start + slope_pos + time_hold + slope_neg)) and (idx < (total_time))):
        # Hold end
        sv_temp = sv_end
        print "\033[12;88H \033[1;33m%s\033[0;39m" % (tec_status[4])
    elif (idx >= total_time):
        sv_temp = sv_start
        print "\033[12;88H \033[1;33m%s\033[0;39m" % (tec_status[6])
        print "\033[40;1H\r\n"
        if (cfg.get('mode', 'no_thermal', 1) == "false"):
    	    tecsmu.off_temp()
	    quit()

    if (cfg.get('mode', 'no_thermal', 1) == "false"):
	tecsmu.set_tmp("%5.3f" % sv_temp)
	time.sleep(0.1)
	pv_temp, tec_curr = tecsmu.get_data()
    else:
	pv_temp = 0.0
	tec_curr = 0.0
    icnt = icnt + 1

    if (cfg.get('mode', 'no_thermal', 1) == "false"):
	tecsmu.set_tmp("%5.3f" % sv_temp)
	tecsmu.set_tmp("%5.3f" % sv_temp)
	tecsmu.set_tmp("%5.3f" % sv_temp)
	tecsmu.set_tmp("%5.3f" % sv_temp)
	time.sleep(0.1)
	pv_temp, tec_curr = tecsmu.get_data()
    else:
	pv_temp = 0.0
	tec_curr = 0.0

    if (cfg.get('mode', 'no_thermal', 1) == "false"):
	tecsmu.set_tmp("%5.3f" % sv_temp)
	time.sleep(0.1)
	pv_temp, tec_curr = tecsmu.get_data()
    else:
	pv_temp = 0.0
	tec_curr = 0.0
    
    #Measurement logic goes here
    if (cfg.get('mode', 'no_thermal', 1) == "false"):
	tecsmu.set_tmp("%5.3f" % sv_temp)
	time.sleep(0.1)
	pv_temp, tec_curr = tecsmu.get_data()
    else:
	pv_temp = 0.0
	tec_curr = 0.0
    meas_val = dmm1.get_data()
    meas_val2 = dmm2.get_data()#dmm2.get_data()
    meas_val3 = dmm3.get_data()
    meas_val4 = 0#dmm4.get_data()
    meas_val5a = dmm3_temp#dmm5.get_data()
    meas_val5b = dmm3_temp#dmm6.get_bdata()
    meas_val6 = 0#dmm5.get_data()#0#(idx < (time_start + slope_pos) )#dmm6.get_adata()
    nvm_temp = 0#dmm6.get_temp()

    sdev_arr.extend([meas_val])
    sdev_arr5.extend([meas_val2])

    #meas_pcb_temp = (float(dmm2.get_data()) - 0.75) / 0.01
    tread = tread - 1
    if (tread == 0):
        dmm1_temp = dmm1.get_temp()
        dmm2_temp = dmm2.get_temp()
        dmm3_temp = dmm3.get_temp()
        dmm4_temp = 24#dmm4.get_temp()
	tread = 20

    print "\033[9;88H \033[0;32m%2.3f %cC\033[0;39m" % (sv_temp, u"\u00b0")
    print "\033[10;88H \033[1;32m%2.3f %cC\033[0;39m" % (pv_temp, u"\u00b0")
    print "\033[11;88H \033[1;35m%5.4f \033[0;39m" % (tec_curr)
    print "\033[10;55H \033[1;38m%11.8f\033[0;39m" % (meas_val)
    print ("\033[16;10H \033[1;32mMedian A= %.9f V     " % np.median(sdev_arr) )
    print ("\033[17;10H \033[1;33m Sdev A = %.3f uV    " % ( np.std(sdev_arr) * 1e6 ) )

    print ("\033[16;50H \033[1;34mMedian K= %.9f V     " % np.median(sdev_arr5) )
    print ("\033[17;50H \033[1;35m Sdev K = %.3f uV    " % ( np.std(sdev_arr5) * 1e6 ) )

    ppm_delta = ((meas_val / reference) - 1) * 1e6
    ppm_delta2 = ((meas_val2 / reference2) - 1) * 1e6
    ppm_delta3 = ((meas_val3 / reference3) - 1) * 1e6
    ppm_delta4 = ((meas_val4 / reference4) - 1) * 1e6
    ppm_delta5a = ((meas_val5a / reference5) - 1) * 1e6
    ppm_delta5b = ((meas_val5b / reference5) - 1) * 1e6
    ppm_delta6 = ((meas_val6 / reference5) - 1) * 1e6
    #print "\033[20;41H \033[0;35m%11.8G \033[0;39m" % (reference)
    print "\033[41;43H\033[1;35m%11.3f\033[0;39m" % (ppm_delta)
    print "\033[42;43H\033[1;35m%11.3f\033[0;39m" % (ppm_delta2)
    print "\033[43;43H\033[1;34m%11.3f\033[0;39m" % (ppm_delta3)
    print "\033[44;43H\033[1;34m%11.3f\033[0;39m" % (ppm_delta4)
    print "\033[46;43H\033[1;34m%11.3f\033[0;39m" % (ppm_delta5a)
    print "\033[47;43H\033[1;34m%11.3f\033[0;39m" % (ppm_delta5b)
    print "\033[48;43H\033[1;34m%11.3f\033[0;39m" % (ppm_delta5b)

    # Data storage logic goes here
    
    with open(fileName4, 'a') as o1: # Open file handles for storage
	o1.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + ("%2.9g;%2.9f;%2.9f;%2.9f;%2.9f;%2.9f;%2.9f;%3.1f;%3.1f;%3.3f;%3.1f;%4.1f;%3.3f;%3.3f;\n" % \
(float(meas_val),float(meas_val2),float(meas_val3),float(meas_val4),float(meas_val5a),float(meas_val5b),float(meas_val6), float(dmm1_temp), float(dmm2_temp),ext_temp,ext_rh,ext_pressure,pv_temp, float(tec_curr)) ) )
        sys.stdout.flush()
        o1.close()
    
    idx+=1                  # Sample index increment
